/* -*- Mode: C; c-file-style: "bsd"; -*- */

/* Matrix Int relation */

#include <math.h>

#include "matrix.h"
#include "number.h"
#include "tree.h"

#include "ourlibc.h"

#include "op_bindings.h"
#include "types_misc.h"

#define BIND_SYM(func,op)    nxp_bind_operator_function(op, NXP_ELEMENT_TYPE_MATRIX, \
                                                        NXP_ELEMENT_TYPE_NUMBER,        \
							func, func,             \
							NXP_OP_BIND_SYMMETRIC,  \
							NULL);

#define BIND_SYM_FULL(f1,f2,op)    nxp_bind_operator_function(op,                    \
							      NXP_ELEMENT_TYPE_MATRIX,    \
							      NXP_ELEMENT_TYPE_NUMBER,       \
							      f1, f2,                \
							      NXP_OP_BIND_SYMMETRIC, \
							      NULL);



static NxpElement* mult(NxpOp op, NxpElement *a, NxpElement *b, 
			NxpEvalContext *context, GError **error) {
    NxpElementMatrix *result;
    NxpElement *el, *tree;
    gint i,j;

    g_return_val_if_fail(NXP_ELEMENT_MATRIX(a), NULL);
    g_return_val_if_fail(NXP_ELEMENT_NUMBER(b), NULL);

    result = NXP_ELEMENT_MATRIX(nxp_element_matrix_new(NXP_ELEMENT_MATRIX(a)->width,
						       NXP_ELEMENT_MATRIX(a)->height,
						       NULL));

    for (i = 0; i < NXP_ELEMENT_MATRIX(a)->width; i++)
	for (j = 0; j < NXP_ELEMENT_MATRIX(a)->height; j++)
	{
	    tree = elem_TIMES(_element_matrix_get(NXP_ELEMENT_MATRIX(a), j,i),
			      NXP_ELEMENT_REF(b));
	    el = nxp_element_eval(tree, context, NULL);

	    NXP_ELEMENT_UNREF(tree);
	    _element_matrix_index(result, j, i) = el;
	}
    return NXP_ELEMENT(result);
}

/* FIXME: this function and previous should be condensed on only one,
   and they must take care of GErrors properly, too!*/
static NxpElement* mult_s(NxpOp op, NxpElement *a, NxpElement *b, 
			NxpEvalContext *context, GError **error) {
    NxpElementMatrix *result;
    NxpElement *el, *tree;
    gint i,j;

    g_return_val_if_fail(NXP_ELEMENT_MATRIX(a), NULL);
    g_return_val_if_fail(b, NULL);
    
    result = NXP_ELEMENT_MATRIX(nxp_element_matrix_new(NXP_ELEMENT_MATRIX(a)->width,
						       NXP_ELEMENT_MATRIX(a)->height,
						       NULL));

    for (i = 0; i < NXP_ELEMENT_MATRIX(a)->width; i++)
	for (j = 0; j < NXP_ELEMENT_MATRIX(a)->height; j++)
	{
	    tree = elem_TIMES(_element_matrix_get(NXP_ELEMENT_MATRIX(a), j,i),
			      NXP_ELEMENT_REF(b));
	    el = nxp_element_simplify(tree, context, NULL);

	    NXP_ELEMENT_UNREF(tree);
	    _element_matrix_index(result, j, i) = el;
	}
    return NXP_ELEMENT(result);
}

static NxpElement* or(NxpOp op, NxpElement *a, NxpElement *b, 
		       NxpEvalContext *context, GError **error) {
	return nxp_element_int_new((NXP_ELEMENT_MATRIX(a)->width * NXP_ELEMENT_MATRIX(a)->height)?
				   1:nxp_element_is_true(b, context, error));
}

static NxpElement* and(NxpOp op, NxpElement *a, NxpElement *b, 
		       NxpEvalContext *context, GError **error) {
	return nxp_element_int_new((NXP_ELEMENT_MATRIX(a)->width * NXP_ELEMENT_MATRIX(a)->height)?
				   nxp_element_is_true(b, context, error):0);
}

void nxp_element_matrix_number_operator_init(void)
{
    // Somamos a cada elemento?
    // BIND_SYM( sum , NXP_OP_PLUS);
    // BIND_SYM( sum , NXP_OP_MINUS);

    // NXP_OP_MINUS_LEFT --> handled on other files

    BIND_SYM_FULL(mult, mult_s, NXP_OP_STAR);

    //    NXP_OP_SLASH
    //    NXP_OP_HAT
    //    NXP_OP_DOT3
    //    NXP_OP_PLUS2_LEFT
    //    NXP_OP_MINUS2_LEFT
    //    NXP_OP_PLUS2_RIGHT
    //    NXP_OP_MINUS2_RIGHT
    //    NXP_OP_GT
    //    NXP_OP_GT_EQ
    //    NXP_OP_LT
    //    NXP_OP_LT_EQ
    //    NXP_OP_EQ2
    //    NXP_OP_EXCL_EQ
    BIND_SYM(and, NXP_OP_SLASH_BSLASH);
    BIND_SYM(or,  NXP_OP_BSLASH_SLASH);
    //    NXP_OP_TILDE_LEFT
    //    NXP_OP_EXCL_RIGHT
    //    NXP_OP_EQ
    //    NXP_OP_COLON_EQ
}



