/* -*- Mode: C; c-file-style: "bsd"; -*- */

/* String Integer relation */

#include <math.h>
#include <string.h>

#include "int.h"
#include "nxp_string.h"
#include "op_bindings.h"
#include "types_misc.h"
#include "tree.h"

#define BIND_SYM(func,op)    nxp_bind_operator_function(op, NXP_ELEMENT_TYPE_STRING, NXP_ELEMENT_TYPE_INT, \
   			                              func, func, NXP_OP_BIND_SYMMETRIC, NULL);
#define BIND_STRING_INT(func,op) nxp_bind_operator_function(op, NXP_ELEMENT_TYPE_STRING, NXP_ELEMENT_TYPE_INT, \
   			                              func, func,  0, NULL);
#define BIND_INT_STRING(func,op) nxp_bind_operator_function(op, NXP_ELEMENT_INT, NXP_ELEMENT_STRING, \
   			                              func, func,  0, NULL);


static NxpElement* or(NxpOp op, NxpElement *a, NxpElement *b, 
		       NxpEvalContext *context, GError **error) {
	return nxp_element_int_new((strlen(NXP_ELEMENT_STRING(a)->string))?1:mpz_cmp_ui(NXP_ELEMENT_INT(b)->value, 0));
}

static NxpElement* and(NxpOp op, NxpElement *a, NxpElement *b, 
		       NxpEvalContext *context, GError **error) {
	return nxp_element_int_new((strlen(NXP_ELEMENT_STRING(a)->string))?mpz_cmp_ui(NXP_ELEMENT_INT(b)->value, 0):0);
}

static NxpElement* mult_them(NxpOp op, NxpElement *a, NxpElement *b, 
			     NxpEvalContext *context, GError **error) {
    NxpElement *r;
    nxp_int_t times = nxp_element_int_get_si(b, error);
    GString* s = g_string_new(NXP_ELEMENT_STRING(a)->string);
    while(--times)
	s = g_string_append(s, NXP_ELEMENT_STRING(a)->string);
    r = nxp_element_string_new(s->str);
    g_string_free(s,FALSE);
    return r;
}

void nxp_element_string_int_operator_init(void)
{
    
    // BIND_SYM(add_them, NXP_OP_PLUS);
    /*r NXP_OP_MINUS_LEFT --> This case is handled under real.c or int.c */
    BIND_SYM(mult_them,NXP_OP_STAR);

    //  BIND_REAL_INT(sub_real_int, NXP_OP_MINUS);
    //  BIND_INT_REAL(sub_int_real, NXP_OP_MINUS);

    // BIND_REAL_INT(div_real_int, NXP_OP_SLASH);
    // BIND_INT_REAL(div_int_real, NXP_OP_SLASH);

    // BIND_REAL_INT(hat_real_int, NXP_OP_HAT);
    // BIND_INT_REAL(hat_int_real, NXP_OP_HAT);

    // NXP_OP_DOT3... TODO
  
    // PLUS2_left/right MINUS2/left/right in real.c or int.c

    // BIND_REAL_INT(gt_real_int, NXP_OP_GT);
    // BIND_INT_REAL(gt_int_real, NXP_OP_GT);

    // BIND_REAL_INT(lt_real_int, NXP_OP_LT);
    // BIND_INT_REAL(lt_int_real, NXP_OP_LT);

    // BIND_REAL_INT(gte_real_int, NXP_OP_GT_EQ);
    // BIND_INT_REAL(gte_int_real, NXP_OP_GT_EQ);

    //BIND_REAL_INT(lte_real_int, NXP_OP_LT_EQ);
    // BIND_INT_REAL(lte_int_real, NXP_OP_LT_EQ);

    // BIND_SYM(equals,NXP_OP_EQ2);
    // BIND_SYM(nequals,NXP_OP_EXCL_EQ);

    BIND_SYM(and,NXP_OP_SLASH_BSLASH);
    BIND_SYM(or, NXP_OP_BSLASH_SLASH);


    // TILDE LEFT is handled under real.c or int.c
    // EXCL_RIGHT is handled under real.c or int.c

    // BIND_SYM(atrib_error, NXP_OP_EQ);
    // BIND_SYM(atrib_error, NXP_OP_COLON_EQ);
}

