/* -*- c -*- */
#include <gsl/gsl_randist.h>
#include "nxp_kernel.h"
#include "gsl.h"


/*** Gaussian Distribution ******************************************************/
static NxpElement *
nxp_gsl_ran_gaussian(NxpElementReal * SIGMA, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_gaussian(rng, nxp_element_r_get_value_as_double(SIGMA)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_gaussian_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_gaussian_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


static NxpElement *
nxp_gsl_ran_gaussian_ratio_method(NxpElementReal * SIGMA, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_gaussian_ratio_method
			     (rng, nxp_element_r_get_value_as_double(SIGMA)));
    GSL_RETURN(el);
}


/*** Gaussian Tail *******************************************************/
static NxpElement *
nxp_gsl_ran_gaussian_tail(NxpElementReal * A, NxpElementReal * SIGMA, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_gaussian_tail
			     (rng, nxp_element_r_get_value_as_double(A), nxp_element_r_get_value_as_double(SIGMA)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_gaussian_tail_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_gaussian_tail_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** Bivariate Gaussian Distribution **************************************/
static NxpElement *
nxp_gsl_ran_bivariate_gaussian(NxpElementReal * SIGMA_X,
			       NxpElementReal * SIGMA_Y,
			       NxpElementReal * RHO, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *v;
    double          X,
                    Y;
    gsl_ran_bivariate_gaussian(rng, nxp_element_r_get_value_as_double(SIGMA_X), nxp_element_r_get_value_as_double(SIGMA_Y),
			       nxp_element_r_get_value_as_double(RHO), &X, &Y);
    v = nxp_element_vector_new(2, NULL);
    NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(v), 0) =
	nxp_element_real_new(X);
    NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(v), 1) =
	nxp_element_real_new(Y);
    GSL_RETURN(v);
}

static NxpElement *nxp_gsl_ran_bivariate_gaussian_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3 ,NxpElementReal *arg4 ,NxpElementReal *arg5, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_bivariate_gaussian_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg4)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg5)));
	GSL_RETURN(nxp_element_real_new(ret));
}



/*** The Exponential Distribution **************************************/
static NxpElement *
nxp_gsl_ran_exponential(NxpElementReal * MU, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_exponential(rng, nxp_element_r_get_value_as_double(MU)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_exponential_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_exponential_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Laplace Distribution **************************************/
static NxpElement *
nxp_gsl_ran_laplace(NxpElementReal * MU, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_laplace(rng, nxp_element_r_get_value_as_double(MU)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_laplace_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_laplace_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Exponential Power Distribution *****************************/
static NxpElement *
nxp_gsl_ran_exppow(NxpElementReal * A, NxpElementReal * B, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_exppow(rng, nxp_element_r_get_value_as_double(A), nxp_element_r_get_value_as_double(B)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_exppow_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_exppow_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Cauchy Distribution *****************************/
static NxpElement *
nxp_gsl_ran_cauchy(NxpElementReal * A, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_cauchy(rng, nxp_element_r_get_value_as_double(A)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_cauchy_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_cauchy_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Rayleigh Distribution *****************************/
static NxpElement *
nxp_gsl_ran_rayleigh(NxpElementReal * A, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_rayleigh(rng, nxp_element_r_get_value_as_double(A)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_rayleigh_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_rayleigh_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Rayleigh Tail Distribution ************************/
static NxpElement *
nxp_gsl_ran_rayleigh_tail(NxpElementReal * A, NxpElementReal * SIGMA, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_rayleigh_tail
			     (rng, nxp_element_r_get_value_as_double(A), nxp_element_r_get_value_as_double(SIGMA)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_rayleigh_tail_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_rayleigh_tail_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Landau Distribution ************************/
static NxpElement *
nxp_gsl_ran_landau(ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el = nxp_element_real_new(gsl_ran_landau(rng));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_landau_pdf(NxpElementReal *arg1, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_landau_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Levy alpha-Stable Distribution ************************/
static NxpElement *
nxp_gsl_ran_levy(NxpElementReal * C, NxpElementReal * ALPHA, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_levy(rng, nxp_element_r_get_value_as_double(C), nxp_element_r_get_value_as_double(ALPHA)));
    GSL_RETURN(el);
}

/*** The Levy skew alpha-Stable Distribution ************************/
static NxpElement *
nxp_gsl_ran_levy_skew(NxpElementReal * C,
		      NxpElementReal * ALPHA, NxpElementReal * BETA, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_levy_skew
			     (rng, nxp_element_r_get_value_as_double(C), nxp_element_r_get_value_as_double(ALPHA), nxp_element_r_get_value_as_double(BETA)));
    GSL_RETURN(el);
}

/*** The Gamma Distribution ************************/
static NxpElement *
nxp_gsl_ran_gamma(NxpElementReal * A, NxpElementReal * B, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_gamma(rng, nxp_element_r_get_value_as_double(A), nxp_element_r_get_value_as_double(B)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_gamma_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_gamma_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Flat (Uniform) Distribution ************************/
static NxpElement *
nxp_gsl_ran_flat(NxpElementReal * A, NxpElementReal * B, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_flat(rng, nxp_element_r_get_value_as_double(A), nxp_element_r_get_value_as_double(B)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_flat_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_flat_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Lognormal Distribution ************************/
static NxpElement *
nxp_gsl_ran_lognormal(NxpElementReal * A, NxpElementReal * B, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_lognormal(rng, nxp_element_r_get_value_as_double(A), nxp_element_r_get_value_as_double(B)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_lognormal_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_lognormal_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Chi-squared Distribution ************************/
static NxpElement *
nxp_gsl_ran_chisq(NxpElementReal * NU, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_chisq(rng, nxp_element_r_get_value_as_double(NU)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_chisq_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_chisq_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The F-distribution ************************/
static NxpElement *
nxp_gsl_ran_fdist(NxpElementReal * NU1, NxpElementReal * NU2, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_fdist(rng, nxp_element_r_get_value_as_double(NU1), nxp_element_r_get_value_as_double(NU2)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_fdist_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_fdist_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The t-distribution ************************/
static NxpElement *
nxp_gsl_ran_tdist(NxpElementReal * NU, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_tdist(rng, nxp_element_r_get_value_as_double(NU)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_tdist_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_tdist_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Beta distribution ************************/
static NxpElement *
nxp_gsl_ran_beta(NxpElementReal * A, NxpElementReal * B, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_beta(rng, nxp_element_r_get_value_as_double(A), nxp_element_r_get_value_as_double(B)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_beta_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_beta_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Logistic distribution ************************/
static NxpElement *
nxp_gsl_ran_logistic(NxpElementReal * A, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_logistic(rng, nxp_element_r_get_value_as_double(A)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_logistic_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_logistic_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Pareto distribution ************************/
static NxpElement *
nxp_gsl_ran_pareto(NxpElementReal * A, NxpElementReal * B, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_pareto(rng, nxp_element_r_get_value_as_double(A), nxp_element_r_get_value_as_double(B)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_pareto_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_pareto_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Spherical Distribution (2D & 3D) **/
static NxpElement *
nxp_gsl_ran_dir_2d(ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *v = nxp_element_vector_new(2, NULL);
    double          X,
                    Y;
    gsl_ran_dir_2d(rng, &X, &Y);
    NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(v), 0) =
	nxp_element_real_new(X);
    NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(v), 1) =
	nxp_element_real_new(Y);
    GSL_RETURN(v);
}

static NxpElement *
nxp_gsl_ran_dir_3d(ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *v = nxp_element_vector_new(3, NULL);
    double          X,
                    Y,
                    Z;
    gsl_ran_dir_3d(rng, &X, &Y, &Z);
    NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(v), 0) =
	nxp_element_real_new(X);
    NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(v), 1) =
	nxp_element_real_new(Y);
    NXP_ELEMENT_VECTOR_ELEM(NXP_ELEMENT_VECTOR(v), 2) =
	nxp_element_real_new(Z);
    GSL_RETURN(v);
}

static NxpElement *
nxp_gsl_ran_dir_nd(NxpElementInt * N, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *v;
    double         *X = g_new(double, nxp_element_r_get_value_as_int(N));
    gsl_ran_dir_nd(rng, nxp_element_r_get_value_as_int(N), X);
    v = nxp_double_buffer_to_vector(X, nxp_element_r_get_value_as_int(N));
    g_free(X);
    GSL_RETURN(v);
}

/*** The Weibull distribution ************************/
static NxpElement *
nxp_gsl_ran_weibull(NxpElementReal * A, NxpElementReal * B, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_weibull(rng, nxp_element_r_get_value_as_double(A), nxp_element_r_get_value_as_double(B)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_weibull_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_weibull_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Gumbel1 distribution ************************/
static NxpElement *
nxp_gsl_ran_gumbel1(NxpElementReal * A, NxpElementReal * B, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_gumbel1(rng, nxp_element_r_get_value_as_double(A), nxp_element_r_get_value_as_double(B)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_gumbel1_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_gumbel1_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Gumbel2 distribution ************************/
static NxpElement *
nxp_gsl_ran_gumbel2(NxpElementReal * A, NxpElementReal * B, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_gumbel2(rng, nxp_element_r_get_value_as_double(A), nxp_element_r_get_value_as_double(B)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_gumbel2_pdf(NxpElementReal *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_gumbel2_pdf(nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg1)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/**[?TODO?]** General Discrete Distributions ******************/

/*** The Poisson Distribution **************************************/
static NxpElement *
nxp_gsl_ran_poisson(NxpElementReal * MU, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_poisson(rng, nxp_element_r_get_value_as_double(MU)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_poisson_pdf(NxpElementInt *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_poisson_pdf(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Bernoulli Distribution **************************************/
static NxpElement *
nxp_gsl_ran_bernoulli(NxpElementReal * P, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_bernoulli(rng, nxp_element_r_get_value_as_double(P)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_bernoulli_pdf(NxpElementInt *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_bernoulli_pdf(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Binomial Distribution **************************************/
static NxpElement *
nxp_gsl_ran_binomial(NxpElementReal * P, NxpElementInt * N, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_binomial(rng, nxp_element_r_get_value_as_double(P), nxp_element_r_get_value_as_int(N)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_binomial_pdf(NxpElementInt *arg1 ,NxpElementReal *arg2 ,NxpElementInt *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_binomial_pdf(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_int_get_si(NXP_ELEMENT_INT(arg3), error));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Negative Binomial Distribution **************************************/
static NxpElement *
nxp_gsl_ran_negative_binomial(NxpElementReal * P, NxpElementReal * N, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_negative_binomial
			     (rng, nxp_element_r_get_value_as_double(P), nxp_element_r_get_value_as_double(N)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_negative_binomial_pdf(NxpElementInt *arg1 ,NxpElementReal *arg2 ,NxpElementReal *arg3, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_negative_binomial_pdf(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg3)));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Pascal Distribution **************************************/
static NxpElement *
nxp_gsl_ran_pascal(NxpElementReal * P, NxpElementInt * N, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_pascal(rng, nxp_element_r_get_value_as_double(P), nxp_element_r_get_value_as_int(N)));
    GSL_RETURN(el);
}

/*** The Geometric Distribution **************************************/
static NxpElement *
nxp_gsl_ran_geometric(NxpElementReal * P, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_geometric(rng, nxp_element_r_get_value_as_double(P)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_geometric_pdf(NxpElementInt *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_geometric_pdf(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}



/*** The Hypergeometric Distribution **************************************/
static NxpElement *
nxp_gsl_ran_hypergeometric(NxpElementInt * N1,
			   NxpElementInt * N2, NxpElementInt * N3, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_hypergeometric
			     (rng, nxp_element_r_get_value_as_int(N1), nxp_element_r_get_value_as_int(N2), nxp_element_r_get_value_as_int(N3)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_hypergeometric_pdf(NxpElementInt *arg1 ,NxpElementInt *arg2 ,NxpElementInt *arg3 ,NxpElementInt *arg4, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_hypergeometric_pdf(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_int_get_si(NXP_ELEMENT_INT(arg2), error) ,nxp_element_int_get_si(NXP_ELEMENT_INT(arg3), error) ,nxp_element_int_get_si(NXP_ELEMENT_INT(arg4), error));
	GSL_RETURN(nxp_element_real_new(ret));
}


/*** The Logarithmic Distribution ************************/
static NxpElement *
nxp_gsl_ran_logarithmic(NxpElementReal * P, ETC)
{
    gsl_rng        *rng = nxp_eval_context_get_user_data(context, "RNG");
    NxpElement     *el =
	nxp_element_real_new(gsl_ran_logarithmic(rng, nxp_element_r_get_value_as_double(P)));
    GSL_RETURN(el);
}

static NxpElement *nxp_gsl_ran_logarithmic_pdf(NxpElementInt *arg1 ,NxpElementReal *arg2, ETC)
{
	nxp_real_t ret;
	ret = gsl_ran_logarithmic_pdf(nxp_element_int_get_si(NXP_ELEMENT_INT(arg1), error) ,nxp_element_r_get_value_as_double(NXP_ELEMENT_R(arg2)));
	GSL_RETURN(nxp_element_real_new(ret));
}



void
nxp_gsl_randist_init(NxpEvalContext * context)
{
    NxpIdentifier  *id;
nxp_name_space_new(context->cwd,"gsl");
nxp_name_space_new(nxp_name_space_find_child(context->cwd,"gsl"),"randomDistributions");
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "gaussian");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_ran_gaussian, NULL, nxp_gsl_ran_gaussian, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "gaussianPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_gaussian_pdf, NULL, nxp_gsl_ran_gaussian_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "gaussianRatio");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_ran_gaussian_ratio_method, NULL, nxp_gsl_ran_gaussian_ratio_method, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "gaussianTail");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_gaussian_tail, NULL, nxp_gsl_ran_gaussian_tail, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "gaussianTailPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_ran_gaussian_tail_pdf, NULL, nxp_gsl_ran_gaussian_tail_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "bivariateGaussian");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_ran_bivariate_gaussian, NULL, nxp_gsl_ran_bivariate_gaussian, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "bivariateGaussianPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL_REAL_REAL, nxp_gsl_ran_bivariate_gaussian_pdf, NULL, nxp_gsl_ran_bivariate_gaussian_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "exponential");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_ran_exponential, NULL, nxp_gsl_ran_exponential, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "exponentialPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_exponential_pdf, NULL, nxp_gsl_ran_exponential_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "laplace");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_ran_laplace, NULL, nxp_gsl_ran_laplace, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "laplacePDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_laplace_pdf, NULL, nxp_gsl_ran_laplace_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "exponentialPower");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_exppow, NULL, nxp_gsl_ran_exppow, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "exponentialPowerPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_ran_exppow_pdf, NULL, nxp_gsl_ran_exppow_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "cauchy");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_ran_cauchy, NULL, nxp_gsl_ran_cauchy, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "cauchyPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_cauchy_pdf, NULL, nxp_gsl_ran_cauchy_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "rayleigh");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_ran_rayleigh, NULL, nxp_gsl_ran_rayleigh, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "rayleighPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_rayleigh_pdf, NULL, nxp_gsl_ran_rayleigh_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "rayleighTail");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_ran_rayleigh_tail, NULL, nxp_gsl_ran_rayleigh_tail, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "rayleighTailPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_ran_rayleigh_tail_pdf, NULL, nxp_gsl_ran_rayleigh_tail_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "landau");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_ran_landau, NULL, nxp_gsl_ran_landau, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "landauPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_ran_landau_pdf, NULL, nxp_gsl_ran_landau_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "levy");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_levy, NULL, nxp_gsl_ran_levy, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "levySkew");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_ran_levy_skew, NULL, nxp_gsl_ran_levy_skew, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "gamma");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_gamma, NULL, nxp_gsl_ran_gamma, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "gammaPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_ran_gamma_pdf, NULL, nxp_gsl_ran_gamma_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "flat");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_flat, NULL, nxp_gsl_ran_flat, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "flatPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_ran_flat_pdf, NULL, nxp_gsl_ran_flat_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "lognormal");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_lognormal, NULL, nxp_gsl_ran_lognormal, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "lognormalPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_ran_lognormal_pdf, NULL, nxp_gsl_ran_lognormal_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "chiSquared");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_chisq, NULL, nxp_gsl_ran_chisq, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "chiSquaredPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_chisq_pdf, NULL, nxp_gsl_ran_chisq_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "fDistribution");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_fdist, NULL, nxp_gsl_ran_fdist, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "fDistributionPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_ran_fdist_pdf, NULL, nxp_gsl_ran_fdist_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "tDistribution");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_ran_tdist, NULL, nxp_gsl_ran_tdist, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "tDistributionPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_tdist_pdf, NULL, nxp_gsl_ran_tdist_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "beta");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_beta, NULL, nxp_gsl_ran_beta, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "betaPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_ran_beta_pdf, NULL, nxp_gsl_ran_beta_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "logistic");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_ran_logistic, NULL, nxp_gsl_ran_logistic, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "logisticPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_logistic_pdf, NULL, nxp_gsl_ran_logistic_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "pareto");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_pareto, NULL, nxp_gsl_ran_pareto, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "paretoPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_ran_pareto_pdf, NULL, nxp_gsl_ran_pareto_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "dir2D");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_ran_dir_2d, NULL, nxp_gsl_ran_dir_2d, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "dir3D");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_ran_dir_3d, NULL, nxp_gsl_ran_dir_3d, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "dirnD");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_ran_dir_nd, NULL, nxp_gsl_ran_dir_nd, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "weibull");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_weibull, NULL, nxp_gsl_ran_weibull, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "weibullPDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_ran_weibull_pdf, NULL, nxp_gsl_ran_weibull_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "gumbelType1");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_gumbel1, NULL, nxp_gsl_ran_gumbel1, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "gumbelType1PDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_ran_gumbel1_pdf, NULL, nxp_gsl_ran_gumbel1_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "gumbelType2");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_gumbel2, NULL, nxp_gsl_ran_gumbel2, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "gumbelType2PDF");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL_REAL, nxp_gsl_ran_gumbel2_pdf, NULL, nxp_gsl_ran_gumbel2_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "poisson");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_ran_poisson, NULL, nxp_gsl_ran_poisson, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "poissonPDF");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_ran_poisson_pdf, NULL, nxp_gsl_ran_poisson_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "bernoulli");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_ran_bernoulli, NULL, nxp_gsl_ran_bernoulli, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "bernoulliPDF");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_ran_bernoulli_pdf, NULL, nxp_gsl_ran_bernoulli_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "binomial");
 nxp_bifunc_define(id, &nxp_prototype__REAL_INT, nxp_gsl_ran_binomial, NULL, nxp_gsl_ran_binomial, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "binomialPDF");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL_INT, nxp_gsl_ran_binomial_pdf, NULL, nxp_gsl_ran_binomial_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "negativeBinomial");
 nxp_bifunc_define(id, &nxp_prototype__REAL_REAL, nxp_gsl_ran_negative_binomial, NULL, nxp_gsl_ran_negative_binomial, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "negativeBinomialPDF");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL_REAL, nxp_gsl_ran_negative_binomial_pdf, NULL, nxp_gsl_ran_negative_binomial_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "pascal");
 nxp_bifunc_define(id, &nxp_prototype__REAL_INT, nxp_gsl_ran_pascal, NULL, nxp_gsl_ran_pascal, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "geometric");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_ran_geometric, NULL, nxp_gsl_ran_geometric, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "geometricPDF");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_ran_geometric_pdf, NULL, nxp_gsl_ran_geometric_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "hypergeometric");
 nxp_bifunc_define(id, &nxp_prototype__INT_INT_INT, nxp_gsl_ran_hypergeometric, NULL, nxp_gsl_ran_hypergeometric, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "hypergeometricPDF");
 nxp_bifunc_define(id, &nxp_prototype__INT_INT_INT_INT, nxp_gsl_ran_hypergeometric_pdf, NULL, nxp_gsl_ran_hypergeometric_pdf, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "logarithmic");
 nxp_bifunc_define(id, &nxp_prototype__REAL, nxp_gsl_ran_logarithmic, NULL, nxp_gsl_ran_logarithmic, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","randomDistributions", "logarithmicPDF");
 nxp_bifunc_define(id, &nxp_prototype__INT_REAL, nxp_gsl_ran_logarithmic_pdf, NULL, nxp_gsl_ran_logarithmic_pdf, context, NULL); nxp_identifier_unref(id);
}
