/* -*- c -*- */
#include <gsl/gsl_statistics_int.h>
#include <gsl/gsl_statistics_double.h>
#include <gsl/gsl_sort.h>
#include "nxp_kernel.h"
#include "gsl.h"
#include "i18n.h"

#define PROPAGATE     if (err) { g_propagate_error(error, err);	return NULL; }
#define G_ERR         GError *err = NULL


static NxpElement *
nxp_gsl_stats_mean(NxpElementVector * v, ETC)
{
    G_ERR;
    NxpElement *answer;

    double *DATA = nxp_vector_to_double_buffer(v, context, &err); PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_mean(DATA, 1, v->size));
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_variance(NxpElementVector * v, ETC)
{
    G_ERR;
    NxpElement *answer;

    double *DATA = nxp_vector_to_double_buffer(v, context, &err); PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_variance(DATA, 1, v->size));
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_variance_m(NxpElementVector * v, NxpElementReal * mean, ETC)
{
    G_ERR;
    NxpElement *answer;

    double *DATA = nxp_vector_to_double_buffer(v, context, &err); PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_variance_m
				  (DATA, 1, v->size, nxp_element_r_get_value_as_double(mean)));
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_sd(NxpElementVector * v, ETC)
{
    G_ERR;
    NxpElement *answer;

    double *DATA = nxp_vector_to_double_buffer(v, context, &err); PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_sd(DATA, 1, v->size));
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_sd_m(NxpElementVector * v, NxpElementReal * mean, ETC)
{
    G_ERR;
    NxpElement *answer;

    double *DATA = nxp_vector_to_double_buffer(v, context, &err); PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_sd_m
				  (DATA, 1, v->size, nxp_element_r_get_value_as_double(mean)));
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_variance_with_fixed_mean(NxpElementVector * v,
				       NxpElementReal * mean, ETC)
{
    G_ERR;
    NxpElement *answer;

    double *DATA = nxp_vector_to_double_buffer(v, context, &err); PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_variance_with_fixed_mean
				  (DATA, 1, v->size, nxp_element_r_get_value_as_double(mean)));
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_sd_with_fixed_mean(NxpElementVector * v,
				 NxpElementReal * mean, ETC)
{
    G_ERR;
    NxpElement *answer;

    double *DATA = nxp_vector_to_double_buffer(v, context, &err); PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_sd_with_fixed_mean
				  (DATA, 1, v->size, nxp_element_r_get_value_as_double(mean)));
    g_free(DATA);
    GSL_RETURN(answer);
}


static NxpElement *
nxp_gsl_stats_absdev(NxpElementVector * v, ETC)
{
    G_ERR;
    NxpElement *answer;

    double *DATA = nxp_vector_to_double_buffer(v, context, &err); PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_absdev(DATA, 1, v->size));
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_absdev_m(NxpElementVector * v, NxpElementReal * m, ETC)
{
    G_ERR;
    NxpElement *answer;

    double *DATA = nxp_vector_to_double_buffer(v, context, &err); PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_absdev_m
				  (DATA, 1, v->size, nxp_element_r_get_value_as_double(m)));
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_skew(NxpElementVector * v, ETC)
{
    G_ERR;
    NxpElement *answer;

    double *DATA = nxp_vector_to_double_buffer(v, context, &err); PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_skew(DATA, 1, v->size));
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_skew_m_sd(NxpElementVector * v,
			NxpElementReal * a, NxpElementReal * b, ETC)
{
    G_ERR;
    NxpElement *answer;

    double *DATA = nxp_vector_to_double_buffer(v, context, &err); PROPAGATE;
    
    answer = nxp_element_real_new(gsl_stats_skew_m_sd
				  (DATA, 1, v->size, nxp_element_r_get_value_as_double(a), nxp_element_r_get_value_as_double(b)));
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_kurtosis(NxpElementVector * v, ETC)
{
    G_ERR;
    NxpElement *answer;

    double *DATA = nxp_vector_to_double_buffer(v, context, &err); PROPAGATE;
								   
    answer = nxp_element_real_new(gsl_stats_kurtosis(DATA, 1, v->size));
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_kurtosis_m_sd(NxpElementVector * v,
			    NxpElementReal * a, NxpElementReal * b, ETC)
{
    G_ERR;
    NxpElement *answer;

    double *DATA = nxp_vector_to_double_buffer(v, context, &err); PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_kurtosis_m_sd
				  (DATA, 1, v->size, nxp_element_r_get_value_as_double(a), nxp_element_r_get_value_as_double(b)));
    g_free(DATA);
    GSL_RETURN(answer);
}


static NxpElement *
nxp_gsl_stats_lag1_autocorrelation(NxpElementVector * v, ETC)
{
    G_ERR;
    NxpElement *answer;
    double *DATA = nxp_vector_to_double_buffer(v, context, &err); PROPAGATE;
    answer = nxp_element_real_new(gsl_stats_lag1_autocorrelation
				  (DATA, 1, v->size));
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_lag1_autocorrelation_m(NxpElementVector * v,
				     NxpElementReal * mean, ETC)
{
    G_ERR;
    NxpElement *answer;
    double *DATA = nxp_vector_to_double_buffer(v, context, &err); PROPAGATE;
    answer = nxp_element_real_new(gsl_stats_lag1_autocorrelation_m
				  (DATA, 1, v->size, nxp_element_r_get_value_as_double(mean)));
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_covariance(NxpElementVector * v1, NxpElementVector * v2, ETC)
{
    G_ERR;
    double         *DATA1;
    double         *DATA2;
    NxpElement     *answer;

    if (v1->size != v2->size) {
	g_set_error(error, nxp_error_domain,
		    NXP_ERROR_TYPE_CHECK, _("Vectors lengths must match"));
	return NULL;
    }

    DATA1 = nxp_vector_to_double_buffer(v1, context, &err); PROPAGATE;
    DATA2 = nxp_vector_to_double_buffer(v2, context, &err); PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_covariance(DATA1, 1,
						       DATA2, 1,
						       v1->size));
    g_free(DATA1);
    g_free(DATA2);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_covariance_m(NxpElementVector * v1, NxpElementReal * m1,
			   NxpElementVector * v2, NxpElementReal * m2, ETC)
{
    G_ERR;
    double         *DATA1;
    double         *DATA2;
    NxpElement     *answer;

    if (v1->size != v2->size) {
	g_set_error(error, nxp_error_domain,
		    NXP_ERROR_TYPE_CHECK, _("Vectors lengths must match"));	
	return NULL;
    }

    DATA1 = nxp_vector_to_double_buffer(v1, context, &err); PROPAGATE;
    DATA2 = nxp_vector_to_double_buffer(v2, context, &err); PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_covariance_m(DATA1, 1,
							 DATA2, 1,
							 v1->size,
							 nxp_element_r_get_value_as_double(m1),
							 nxp_element_r_get_value_as_double(m2)));
    g_free(DATA1);
    g_free(DATA2);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_wmean(NxpElementVector * wv, NxpElementVector * v, ETC)
{
    G_ERR;
    double         *WDATA;
    double         *DATA;
    NxpElement     *answer;

    if (wv->size != v->size) {
	g_set_error(error, nxp_error_domain,
		    NXP_ERROR_TYPE_CHECK, _("Vectors lengths must match"));
	return NULL;
    }

    WDATA = nxp_vector_to_double_buffer(wv, context, &err); PROPAGATE;
    DATA = nxp_vector_to_double_buffer(v, context, &err);   PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_wmean(WDATA, 1,
						  DATA, 1, v->size));
    g_free(WDATA);
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_wvariance(NxpElementVector * wv, NxpElementVector * v, ETC)
{
    G_ERR;
    double         *WDATA;
    double         *DATA;
    NxpElement     *answer;

    if (wv->size != v->size) {
	g_set_error(error, nxp_error_domain,
		    NXP_ERROR_TYPE_CHECK, _("Weight and data vectors length must match"));
	return NULL;
    }

    WDATA = nxp_vector_to_double_buffer(wv, context, &err); PROPAGATE;
    DATA = nxp_vector_to_double_buffer(v, context, &err);   PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_wvariance(WDATA, 1,
						      DATA, 1, v->size));
    g_free(WDATA);
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_wvariance_m(NxpElementVector * wv,
			  NxpElementVector * v, NxpElementReal * wm, ETC)
{
    G_ERR;
    double         *WDATA;
    double         *DATA;
    NxpElement     *answer;

    if (wv->size != v->size) {
	g_set_error(error, nxp_error_domain,
		    NXP_ERROR_TYPE_CHECK, _("Weight and data vectors length must match"));
	return NULL;
    }

    WDATA = nxp_vector_to_double_buffer(wv, context, &err); PROPAGATE;
    DATA = nxp_vector_to_double_buffer(v, context, &err);   PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_wvariance_m(WDATA, 1,
							DATA, 1, v->size,
							nxp_element_r_get_value_as_double(wm)));
    g_free(WDATA);
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_wsd(NxpElementVector * wv, NxpElementVector * v, ETC)
{
    G_ERR;
    double         *WDATA;
    double         *DATA;
    NxpElement     *answer;

    if (wv->size != v->size) {
	g_set_error(error, nxp_error_domain,
		    NXP_ERROR_TYPE_CHECK, _("Weight and data vectors length must match"));
	return NULL;
    }

    WDATA = nxp_vector_to_double_buffer(wv, context, &err); PROPAGATE;
    DATA = nxp_vector_to_double_buffer(v, context, &err);   PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_wsd(WDATA, 1,
						DATA, 1, v->size));
    g_free(WDATA);
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_wsd_m(NxpElementVector * wv,
		    NxpElementVector * v, NxpElementReal * wm, ETC)
{
    G_ERR;
    double         *WDATA;
    double         *DATA;
    NxpElement     *answer;

    if (wv->size != v->size) {
	g_set_error(error, nxp_error_domain,
		    NXP_ERROR_TYPE_CHECK, _("Weight and data vectors length must match"));
	return NULL;
    }

    WDATA = nxp_vector_to_double_buffer(wv, context, &err); PROPAGATE;
    DATA = nxp_vector_to_double_buffer(v, context, &err);   PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_wsd_m(WDATA, 1,
						  DATA, 1, v->size,
						  nxp_element_r_get_value_as_double(wm)));
    g_free(WDATA);
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_wvariance_with_fixed_mean(NxpElementVector * weight,
					NxpElementVector * data,
					NxpElementReal * wmean, ETC)
{
    G_ERR;
    double         *WDATA;
    double         *DATA;
    NxpElement     *answer;

    if (weight->size != data->size) {
	g_set_error(error, nxp_error_domain,
		    NXP_ERROR_TYPE_CHECK, _("Weight and data vectors length must match"));
	return NULL;
    }

    WDATA = nxp_vector_to_double_buffer(weight, context, &err); PROPAGATE;
    DATA = nxp_vector_to_double_buffer(data, context, &err);    PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_wvariance_with_fixed_mean
				  (WDATA, 1, DATA, 1, data->size,
				   nxp_element_r_get_value_as_double(wmean)));
    g_free(WDATA);
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_wsd_with_fixed_mean(NxpElementVector * weight,
				  NxpElementVector * data,
				  NxpElementReal * wmean, ETC)
{
    G_ERR;
    double         *WDATA;
    double         *DATA;
    NxpElement     *answer;

    if (weight->size != data->size) {
	g_set_error(error, nxp_error_domain,
		    NXP_ERROR_TYPE_CHECK, _("Weight and data vectors length must match"));
	return NULL;
    }
    WDATA = nxp_vector_to_double_buffer(weight, context, &err); PROPAGATE;
    DATA = nxp_vector_to_double_buffer(data, context, &err);    PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_wsd_with_fixed_mean(WDATA, 1,
								DATA, 1,
								data->size,
								nxp_element_r_get_value_as_double(NXP_ELEMENT_R(wmean))));
    g_free(WDATA);
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_wabsdev(NxpElementVector * weight,
		      NxpElementVector * data, ETC)
{
    G_ERR;
    double         *WDATA;
    double         *DATA;
    NxpElement     *answer;

    if (weight->size != data->size) {
	g_set_error(error, nxp_error_domain,
		    NXP_ERROR_TYPE_CHECK, _("Weight and data vectors length must match"));
	return NULL;		
    }

    WDATA = nxp_vector_to_double_buffer(weight, context, &err); PROPAGATE;
    DATA = nxp_vector_to_double_buffer(data, context, &err);    PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_wabsdev(WDATA, 1,
						    DATA, 1, data->size));
    g_free(WDATA);
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_wabsdev_m(NxpElementVector * weight,
			NxpElementVector * data,
			NxpElementReal * wmean, ETC)
{    
    G_ERR;
    double         *WDATA;
    double         *DATA;
    NxpElement     *answer;

    if (weight->size != data->size) {
	g_set_error(error, nxp_error_domain,
		    NXP_ERROR_TYPE_CHECK, _("Weight and data vectors length must match"));
	return NULL;
    }

    WDATA = nxp_vector_to_double_buffer(weight, context, &err); PROPAGATE;
    DATA = nxp_vector_to_double_buffer(data, context, &err);    PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_wabsdev_m(WDATA, 1,
						      DATA, 1, data->size,
						      nxp_element_r_get_value_as_double(wmean)));
    g_free(WDATA);
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_wskew(NxpElementVector * weight,
		    NxpElementVector * data, ETC)
{
    G_ERR;
    double         *WDATA;
    double         *DATA;
    NxpElement     *answer;

    if (weight->size != data->size) {
	g_set_error(error, nxp_error_domain,
		    NXP_ERROR_TYPE_CHECK, _("Weight and data vectors length must match"));
	return NULL;
    }

    WDATA = nxp_vector_to_double_buffer(weight, context, &err); PROPAGATE;
    DATA = nxp_vector_to_double_buffer(data, context, &err);    PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_wskew(WDATA, 1,
						  DATA, 1, data->size));
    g_free(WDATA);
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_wskew_m_sd(NxpElementVector * weight,
			 NxpElementVector * data,
			 NxpElementReal * wmean, NxpElementReal * wsd, ETC)
{
    G_ERR;
    double         *WDATA;
    double         *DATA;
    NxpElement     *answer;

    if (weight->size != data->size) {
	g_set_error(error, nxp_error_domain,
		    NXP_ERROR_TYPE_CHECK, _("Weight and data vectors length must match"));
	return NULL;
    }

    WDATA = nxp_vector_to_double_buffer(weight, context, &err); PROPAGATE;
    DATA = nxp_vector_to_double_buffer(data, context, &err);    PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_wskew_m_sd(WDATA, 1,
						       DATA, 1, data->size,
						       nxp_element_r_get_value_as_double(wmean),
						       nxp_element_r_get_value_as_double(wsd)));
    g_free(WDATA);
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_wkurtosis(NxpElementVector * weight,
			NxpElementVector * data, ETC)
{
    G_ERR;
    double         *WDATA;
    double         *DATA;
    NxpElement     *answer;

    if (weight->size != data->size) {
	g_set_error(error, nxp_error_domain,
		    NXP_ERROR_TYPE_CHECK, _("Weight and data vectors length must match"));
	return NULL;
    }

    WDATA = nxp_vector_to_double_buffer(weight, context, &err); PROPAGATE;
    DATA = nxp_vector_to_double_buffer(data, context, &err);    PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_wkurtosis(WDATA, 1,
						      DATA, 1,
						      data->size));
    g_free(WDATA);
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_wkurtosis_m_sd(NxpElementVector * weight,
			     NxpElementVector * data,
			     NxpElementReal * wmean,
			     NxpElementReal * wsd, ETC)
{
    G_ERR;
    double         *WDATA;
    double         *DATA;
    NxpElement     *answer;

    if (weight->size != data->size) {
	g_set_error(error, nxp_error_domain,
		    NXP_ERROR_TYPE_CHECK, _("Weight and data vector lengths must match"));
	return NULL;
    }

    WDATA = nxp_vector_to_double_buffer(weight, context, &err); PROPAGATE;
    DATA = nxp_vector_to_double_buffer(data, context, &err);    PROPAGATE;

    answer = nxp_element_real_new(gsl_stats_wkurtosis_m_sd(WDATA, 1,
							   DATA, 1,
							   data->size,
							   nxp_element_r_get_value_as_double(wmean),
							   nxp_element_r_get_value_as_double(wsd)));
    g_free(WDATA);
    g_free(DATA);
    GSL_RETURN(answer);
}


/* MEDIAN AND PERCENTILES */
static NxpElement *
nxp_gsl_stats_median_from_sorted_data(NxpElementVector * data, ETC)
{
    G_ERR;
    double         *DATA;
    NxpElement     *answer;
    DATA = nxp_vector_to_double_buffer(data, context, &err); PROPAGATE;
    gsl_sort(DATA, 1, data->size);
    answer =
	nxp_element_real_new(gsl_stats_median_from_sorted_data
			     (DATA, 1, data->size));
    g_free(DATA);
    GSL_RETURN(answer);
}

static NxpElement *
nxp_gsl_stats_quantile_from_sorted_data(NxpElementVector * data,
					NxpElementReal * frac, ETC)
{
    G_ERR;
    double         *DATA;
    NxpElement     *answer;
    DATA = nxp_vector_to_double_buffer(data, context, &err); PROPAGATE;
    gsl_sort(DATA, 1, data->size);
    answer = nxp_element_real_new(gsl_stats_quantile_from_sorted_data
				  (DATA, 1, data->size, nxp_element_r_get_value_as_double(frac)));
    g_free(DATA);
    GSL_RETURN(answer);
}

void
nxp_gsl_stats_init(NxpEvalContext * context)
{
    NxpIdentifier  *id;
nxp_name_space_new(context->cwd,"gsl");
nxp_name_space_new(nxp_name_space_find_child(context->cwd,"gsl"),"statistics");
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "mean");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR, nxp_gsl_stats_mean, NULL, nxp_gsl_stats_mean, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "variance");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR, nxp_gsl_stats_variance, NULL, nxp_gsl_stats_variance, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "variance");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_REAL, nxp_gsl_stats_variance_m, NULL, nxp_gsl_stats_variance_m, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "sd");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR, nxp_gsl_stats_sd, NULL, nxp_gsl_stats_sd, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "sd");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_REAL, nxp_gsl_stats_sd_m, NULL, nxp_gsl_stats_sd_m, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "varianceFxdMean");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_REAL, nxp_gsl_stats_variance_with_fixed_mean, NULL, nxp_gsl_stats_variance_with_fixed_mean, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "sdFxdMean");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_REAL, nxp_gsl_stats_sd_with_fixed_mean, NULL, nxp_gsl_stats_sd_with_fixed_mean, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "absDev");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR, nxp_gsl_stats_absdev, NULL, nxp_gsl_stats_absdev, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "absDev");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_REAL, nxp_gsl_stats_absdev_m, NULL, nxp_gsl_stats_absdev_m, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "skew");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR, nxp_gsl_stats_skew, NULL, nxp_gsl_stats_skew, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "skew");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_REAL_REAL, nxp_gsl_stats_skew_m_sd, NULL, nxp_gsl_stats_skew_m_sd, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "kurtosis");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR, nxp_gsl_stats_kurtosis, NULL, nxp_gsl_stats_kurtosis, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "kurtosis");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_REAL_REAL, nxp_gsl_stats_kurtosis_m_sd, NULL, nxp_gsl_stats_kurtosis_m_sd, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "lag1Autocorrelation");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR, nxp_gsl_stats_lag1_autocorrelation, NULL, nxp_gsl_stats_lag1_autocorrelation, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "lag1Autocorrelation");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_REAL, nxp_gsl_stats_lag1_autocorrelation_m, NULL, nxp_gsl_stats_lag1_autocorrelation_m, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "covariance");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_VECTOR, nxp_gsl_stats_covariance, NULL, nxp_gsl_stats_covariance, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "covariance");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_REAL_VECTOR_REAL, nxp_gsl_stats_covariance_m, NULL, nxp_gsl_stats_covariance_m, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "wmean");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_VECTOR, nxp_gsl_stats_wmean, NULL, nxp_gsl_stats_wmean, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "wvariance");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_VECTOR, nxp_gsl_stats_wvariance, NULL, nxp_gsl_stats_wvariance, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "wvariance");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_VECTOR_REAL, nxp_gsl_stats_wvariance_m, NULL, nxp_gsl_stats_wvariance_m, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "wsd");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_VECTOR, nxp_gsl_stats_wsd, NULL, nxp_gsl_stats_wsd, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "wsd");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_VECTOR_REAL, nxp_gsl_stats_wsd_m, NULL, nxp_gsl_stats_wsd_m, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "wvarianceFxdMean");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_VECTOR_REAL, nxp_gsl_stats_wvariance_with_fixed_mean, NULL, nxp_gsl_stats_wvariance_with_fixed_mean, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "wsdFxdMean");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_VECTOR_REAL, nxp_gsl_stats_wsd_with_fixed_mean, NULL, nxp_gsl_stats_wsd_with_fixed_mean, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "wabsDev");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_VECTOR, nxp_gsl_stats_wabsdev, NULL, nxp_gsl_stats_wabsdev, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "wabsDev");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_VECTOR_REAL, nxp_gsl_stats_wabsdev_m, NULL, nxp_gsl_stats_wabsdev_m, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "wskew");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_VECTOR, nxp_gsl_stats_wskew, NULL, nxp_gsl_stats_wskew, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "wskew");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_VECTOR_REAL_REAL, nxp_gsl_stats_wskew_m_sd, NULL, nxp_gsl_stats_wskew_m_sd, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "wkurtosis");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_VECTOR, nxp_gsl_stats_wkurtosis, NULL, nxp_gsl_stats_wkurtosis, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "wkurtosis");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_VECTOR_REAL_REAL, nxp_gsl_stats_wkurtosis_m_sd, NULL, nxp_gsl_stats_wkurtosis_m_sd, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "median");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR, nxp_gsl_stats_median_from_sorted_data, NULL, nxp_gsl_stats_median_from_sorted_data, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","statistics", "quantile");
 nxp_bifunc_define(id, &nxp_prototype__VECTOR_REAL, nxp_gsl_stats_quantile_from_sorted_data, NULL, nxp_gsl_stats_quantile_from_sorted_data, context, NULL); nxp_identifier_unref(id);
}

#undef PROPAGATE
#undef G_ERR
