#include "nxp_kernel.h"
#include "functions.h"

#include "config.h"

NxpElement* stdlib_eval_re(NxpElementComplex *el, ETC)
{
    return NXP_ELEMENT_REF(NXP_COMPLEX_REAL(el));
}

NxpElement* stdlib_eval_im(NxpElementComplex *el, ETC)
{
    return NXP_ELEMENT_REF(NXP_COMPLEX_IMG(el));
}

NxpElement* stdlib_eval_abs(NxpElementComplex *el, ETC)
{
    mpfr_t result;
    NxpElement *r, *i;

    if (!NXP_ELEMENT_IS_REAL(NXP_COMPLEX_REAL(el)))
	r = nxp_typecasts_try_cast(NXP_COMPLEX_REAL(el), NXP_ELEMENT_TYPE_REAL);
    else
	r = NXP_ELEMENT_REF(NXP_COMPLEX_REAL(el));

    if (!NXP_ELEMENT_IS_REAL(NXP_COMPLEX_IMG(el)))
	i = nxp_typecasts_try_cast(NXP_COMPLEX_IMG(el), NXP_ELEMENT_TYPE_REAL);
    else
	i = NXP_ELEMENT_REF(NXP_COMPLEX_IMG(el));

    mpfr_init(result);
    mpfr_hypot(result, NXP_ELEMENT_REAL(r)->value, NXP_ELEMENT_REAL(i)->value,
	       GMP_RNDN);
    NXP_ELEMENT_UNREF(r);
    NXP_ELEMENT_UNREF(i);
    return nxp_element_real_new_from_mpfr(result);
}

NxpElement* stdlib_eval_arg(NxpElementComplex *el, ETC)
{
    mpfr_t result;
    NxpElement *r, *i;

    if (!NXP_ELEMENT_IS_REAL(NXP_COMPLEX_REAL(el)))
	r = nxp_typecasts_try_cast(NXP_COMPLEX_REAL(el), NXP_ELEMENT_TYPE_REAL);
    else
	r = NXP_ELEMENT_REF(NXP_COMPLEX_REAL(el));

    if (!NXP_ELEMENT_IS_REAL(NXP_COMPLEX_IMG(el)))
	i = nxp_typecasts_try_cast(NXP_COMPLEX_IMG(el), NXP_ELEMENT_TYPE_REAL);
    else
	i = NXP_ELEMENT_REF(NXP_COMPLEX_IMG(el));

    mpfr_init(result);
    mpfr_atan2(result, NXP_ELEMENT_REAL(i)->value, NXP_ELEMENT_REAL(r)->value, GMP_RNDN);
    NXP_ELEMENT_UNREF(r);
    NXP_ELEMENT_UNREF(i);
    return nxp_element_real_new_from_mpfr(result);
}

NxpElement* stdlib_eval_conjugate(NxpElementComplex *el, ETC)
{
    NxpElement *img1, *retval;

    img1 = nxp_element_r_opposed(NXP_COMPLEX_IMG(el));
    retval = nxp_element_complex_new(NXP_COMPLEX_REAL(el), img1);
    NXP_ELEMENT_UNREF(img1);
    return retval;
}
