#include "nxp_kernel.h"
#include "functions.h"

/* 
   - bifuncs_logic.c -
   ------------------
    And[] - n arguments
    Or[]  - n arguments
    Not[] - 1 argument
*/

NxpElement* stdlib_eval_not(NxpElement* el, ETC)
{
    GError *err = NULL;
    NxpElement *el1;
    gboolean is_true;

    el1 = nxp_element_eval(el, context, &err);
    if (err) {
	g_propagate_error(error, err);
	return NULL;
    }
    is_true = nxp_element_is_true(el1,  context, &err);
    NXP_ELEMENT_UNREF(el1);
    if (err) {
	g_propagate_error(error, err);
	return NULL;
    }

    return nxp_element_int_new(is_true == 0);
} 

NxpElement* stdlib_simp_not(NxpElement* el, ETC)
{
    GError *err = NULL;
    NxpElement *el1;
    gboolean is_true;

    el1 = nxp_element_simplify(el, context, &err);
    if (err) {
	g_propagate_error(error, err);
	return NULL;
    }
    is_true = nxp_element_is_true(el1,  context, &err);
    NXP_ELEMENT_UNREF(el1);
    if (err) {
	g_propagate_error(error, err);
	return NULL;
    }

    return nxp_element_int_new(is_true == 0);
}

static NxpElement* stdlib_and(GSList* args, gboolean eval, ETC)
{ 
    GError *err = NULL;
    GSList *l;
    NxpElement *el;
    gboolean is_true;

    l = args;
    do {
	if (eval)
	    el = nxp_element_eval(l->data, context, &err);
	else
	    el = nxp_element_simplify(l->data, context, &err);
	if (err) {
	    g_propagate_error(error, err);
	    return NULL;
	}

	is_true = nxp_element_is_true(el, context, &err);
	if (err) {
	    g_propagate_error(error, err);
	    NXP_ELEMENT_UNREF(el);
	    return NULL;
	}

	if (!is_true) {
	    NXP_ELEMENT_UNREF(el);
	    return nxp_element_int_new(0);
	}
	l = g_slist_next(l);
	NXP_ELEMENT_UNREF(el);
    }
    while(l);
    return nxp_element_int_new(1);
}

NxpElement* stdlib_eval_and(GSList *args, ETC)
{
    return stdlib_and(args, TRUE, NULL, context, error);
}

NxpElement* stdlib_simp_and(GSList *args, ETC)
{
    return stdlib_and(args, FALSE, NULL, context, error);
}

static NxpElement* stdlib_or(GSList* args, gboolean eval, ETC)
{
    GError *err = NULL;
    GSList *l;
    NxpElement *el;
    gboolean is_true;

    l = args;
    do {
	if (eval)
	    el = nxp_element_eval(l->data, context, &err);
	else
	    el = nxp_element_simplify(l->data, context, &err);
	if (err && !*error) {
	    g_propagate_error(error, err);
	}
	else
	    g_clear_error(&err);

	is_true = nxp_element_is_true(el, context, &err);

	if (is_true) {
	    NXP_ELEMENT_UNREF(el);
	    if (*error)
		g_clear_error(error);
	    return nxp_element_int_new(1);
	}
	l = g_slist_next(l);
	NXP_ELEMENT_UNREF(el);
    }
    while(l);
    if (error)
	return NULL;
    return nxp_element_int_new(0);
}

NxpElement* stdlib_eval_or(GSList *args, ETC)
{
    return stdlib_or(args, TRUE, NULL, context, error);
}

NxpElement* stdlib_simp_or(GSList *args, ETC)
{
    return stdlib_or(args, FALSE, NULL, context, error);
}

