#include <stdio.h>

#include "nxp_kernel.h"
#include "functions.h"
#include "config.h"
#include "i18n.h"

#ifdef HAVE_COMPLEX_H
#include <complex.h>
#endif /* HAVE_COMPLEX_H */

#ifndef HAVE_CLOG10
double complex clog10(double complex z) {
    return clog(z)/log(10.0);
}
#endif

NxpElement* stdlib_eval_real_sqrt(NxpElementReal* el, ETC) {
    mpfr_t res;
    mpfr_init(res);
    if (mpfr_sgn(el->value) >= 0) {
	mpfr_sqrt(res, el->value, GMP_RNDN);
	return nxp_element_real_new_from_mpfr(res);
    }
    else {
	NxpElement *r = nxp_element_int_new(0);
	NxpElement *i, *retval;
	mpfr_neg(res, el->value, GMP_RNDN);
	mpfr_sqrt(res, res, GMP_RNDN);
	i = nxp_element_real_new_from_mpfr(res);
	retval = nxp_element_complex_new(r, i);
	NXP_ELEMENT_UNREF(r);
	NXP_ELEMENT_UNREF(i);
	return retval;
    }
}

NxpElement* stdlib_eval_real_log(NxpElementReal* el, GError **error) {

    if (mpfr_sgn(el->value) > 0) {
	mpfr_t result;

	mpfr_init(result);
	mpfr_log10(result, el->value, GMP_RNDN);
	return nxp_element_real_new_from_mpfr(result);
    }
    else {
	nxp_real_t value = mpfr_get_d(el->value, GMP_RNDN);

	return nxp_element_complex_new_from_dbl(clog10((nxp_complex_t)value));
    }
}


#ifdef HAVE_COMPLEX_H
NxpElement* stdlib_eval_complex_sqrt(NxpElementComplex* el, ETC) {
    return nxp_element_complex_new_from_dbl(csqrt(nxp_element_complex_get_value(el)));
}
#endif

#define EVAL_MPFR(op,func)  NxpElement* stdlib_eval_real_##op(NxpElementReal *el, GError **error) {\
    mpfr_t result;\
    mpfr_init(result);\
    func(result, el->value, GMP_RNDN);\
    return nxp_element_real_new_from_mpfr(result);\
}

#define EVAL(op,func)  NxpElement* stdlib_eval_real_##op(NxpElementR *el, \
                                                            GHashTable *named_args, \
							NxpEvalContext *context, GError **error) {  \
                                 return nxp_element_real_new(func(nxp_element_r_get_value_as_double(el)));}

#ifdef HAVE_COMPLEX_H
#define EVAC(op,func)  NxpElement* stdlib_eval_complex_##op(NxpElementComplex *el, \
                                                            GHashTable *named_args, \
                                                            NxpEvalContext *context, \
                                                                GError **error) {  \
                                  return nxp_element_complex_new_from_dbl(func(nxp_element_complex_get_value(el)));}  
#else
#define EVAC(op,func)  ;
#endif

EVAL_MPFR(exp, mpfr_exp);
EVAC(exp, cexp);

EVAL_MPFR(erf, mpfr_erf);

EVAL_MPFR(erfc, mpfr_erfc);

#undef EVAL
#undef EVAC

#ifdef HAVE_COMPLEX_H
NxpElement* stdlib_eval_complex_log(NxpElementComplex* el, ETC) {
    return nxp_element_complex_new_from_dbl(clog10(nxp_element_complex_get_value(el)));
}
#endif

NxpElement* stdlib_derive_log(NxpIdentifier *var, NxpElement* el, ETC)
{
    return elem_OVER(nxp_element_derive(el, var, context, NULL),
  		     elem_TIMES(NXP_ELEMENT_REF(el),
  				elem_LN(nxp_element_real_new(10))));
}


NxpElement* stdlib_derive_sqrt(NxpIdentifier *var, NxpElement* el, ETC)
{
    return elem_OVER(nxp_element_derive(el, var, context, NULL),
  		     elem_TIMES(nxp_element_real_new(2),
  				elem_SQRT(NXP_ELEMENT_REF(el))));
}
NxpElement* stdlib_derive_exp(NxpIdentifier *var, NxpElement* el, ETC)
{
    return elem_TIMES(elem_EXP(NXP_ELEMENT_REF(el)),
  		      nxp_element_derive(el, var, context, NULL));
}

