/* -*- Mode: C; c-file-style: "bsd"; -*- */
#include <string.h>
#include <glib.h>
#include "parser.h"
#include "message.h"
#include "plugins.h"


/* the prefix for this module is 'exp', not 'exp_parser'  */
static pthread_mutex_t exp_parser_mutex;

char *exp_input;		/* Apontador para o endereco de
                                   memoria da string a fazer parse.*/
char *exp_inputptr;		/* Apontador para o endereco da
                                   posicao actual em *exp_parser_input.*/
int exp_inputlim;		/* Tamanho da string guardada em
				 *exp_parser_input.*/
NxpElement *exp_expression;	/* Variavel global com a arvore
                                   devolvida pelo parser.*/
gchar *exp_error = NULL;
static NxpEvalContext *parser_context;

void experror(char *s) 
{
    exp_expression = NULL;
    if (exp_error) g_free(exp_error);
    exp_error = g_strdup(s);
}

void exp_parser_init(void)
{
    pthread_mutex_init(&exp_parser_mutex, NULL);
    nxp_parser_init();
}



NxpElement* exp_parse_string(const gchar *expr, NxpEvalContext *context, GError **error)
{
    char *ptr;
    NxpElement *tree;

    pthread_mutex_lock(&exp_parser_mutex);
    parser_context = context;
    if (exp_error) g_free(exp_error);
    exp_error = NULL;
    if (exp_input)
	g_free(exp_input);
    ptr = g_strdup(expr);

    /* Albie: s/\n\t/ /g; */
/*    ptr = g_strdelimit(ptr, "\t\n", ' ');*/

    exp_input = (char *) g_malloc(strlen(ptr) + 2);
    strcpy(exp_input, ptr);
    g_free(ptr);
    strcat(exp_input, "\n");
    //--
    exp_inputptr = exp_input;
    exp_inputlim = strlen(exp_input);
    exprestart(NULL);
    expparse();
    tree = exp_expression;
    exp_expression = NULL;

    if (exp_error || tree == NULL) {
	g_set_error(error, EXP_PARSE_ERROR_DOMAIN, EXP_PARSE_ERROR,
		    "%s", exp_error? exp_error : "unknown parse error");
	g_free(exp_error);
	exp_error = NULL;
    }
    parser_context = NULL;
    pthread_mutex_unlock(&exp_parser_mutex);

    return tree;
}


int exp_yyinput(char *buf, int max_size)
{
    int n;

    n = MIN(max_size, exp_input + exp_inputlim - exp_inputptr);
    if (n > 0) 
    {
	memcpy(buf, exp_inputptr, n);
	exp_inputptr += n;
    }
    return n; 
} 


GQuark exp_parse_error_quark(void)
{
    static GQuark q = 0;
    if (q == 0)
	q = g_quark_from_static_string("exp-parser-error");
    return q;
}



void nxp_parser_error(const char *format, ...)
{
    va_list args;
    gchar *str;
    va_start(args, format);
    str = g_strdup_vprintf(format, args);
    if (exp_error) {
	g_warning("Parser error overrun");
	g_free(exp_error);
    }
    exp_error = str;
    va_end(args);
}


void nxp_parser_warning(const char *format, ...)
{
    va_list args;
    gchar *str;
    va_start(args, format);
    str = g_strdup_vprintf(format, args);
    if (parser_context->output_handler)
	parser_context->output_handler(str, NXP_MESSAGE_PARSER_WARNING, parser_context);
    else
	printf("Parser-warning: %s\n", str);
    g_free(str);
    va_end(args);
}


